@@ -7,10 +7,14 @@
 
  To be included by aaip_0_2.c
 
- Copyright (c) 2009 Thomas Schmitt, libburnia project, GPLv2
+ Copyright (c) 2009 Thomas Schmitt, libburnia project, GPLv2+
 
 */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include <ctype.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -291,9 +295,9 @@ ex:;
    aaip_get_acl_text("", &a_acl_text, 1 << 15); /* free */
  if(d_acl_text != NULL)
    aaip_get_acl_text("", &d_acl_text, 1 << 15); /* free */
+ if(list != NULL)
+   free(list);
  if(ret <= 0 || (flag & (1 << 15))) {
-   if(list != NULL)
-     free(list);
    if(*names != NULL) {
      for(i= 0; i < *num_attrs; i++)
        free((*names)[i]);
@@ -397,7 +401,7 @@ int aaip_set_attr_list(char *path, size_t num_attrs, char **names,
  size_t i, consumed, acl_text_fill, acl_idx= 0, h_consumed;
  char *acl_text= NULL, *list= NULL;
 #ifdef Libisofs_with_aaip_xattR
- size_t list_size= 0;
+ ssize_t list_size= 0;
 #endif
 
 #ifdef Libisofs_with_aaip_xattR
