@@ -3,10 +3,15 @@
  * Copyright (c) 2009 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License version 2 as 
- * published by the Free Software Foundation. See COPYING file for details.
+ * modify it under the terms of the GNU General Public License version 2 
+ * or later as published by the Free Software Foundation. 
+ * See COPYING file for details.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 /* libisofs.h defines aaip_xinfo_func */
 #include "libisofs.h"
 
@@ -19,11 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-
-
-#ifndef PATH_MAX
-#define PATH_MAX Libisofs_default_path_maX
-#endif
+#include <stdio.h>
 
 
 
@@ -70,6 +71,8 @@ int default_create_file(IsoNodeBuilder *builder, IsoImage *image,
     iso_file_source_ref(src);
     
     name = iso_file_source_get_name(src);
+    if (strlen(name) > LIBISOFS_NODE_NAME_MAX)
+        name[LIBISOFS_NODE_NAME_MAX] = 0;
     ret = iso_node_new_file(name, stream, &node);
     if (ret < 0) {
         iso_stream_unref(stream);
@@ -99,7 +102,7 @@ int default_create_node(IsoNodeBuilder *builder, IsoImage *image,
     IsoNode *new;
     IsoFilesystem *fs;
     char *name;
-    unsigned char *aa_string;
+    unsigned char *aa_string = NULL;
     char *a_text = NULL, *d_text = NULL;
 
     if (builder == NULL || src == NULL || node == NULL) {
@@ -117,6 +120,8 @@ int default_create_node(IsoNodeBuilder *builder, IsoImage *image,
     }
 
     name = iso_file_source_get_name(src);
+    if (strlen(name) > LIBISOFS_NODE_NAME_MAX)
+        name[LIBISOFS_NODE_NAME_MAX] = 0;
     fs = iso_file_source_get_filesystem(src);
     new = NULL;
 
@@ -152,10 +157,10 @@ int default_create_node(IsoNodeBuilder *builder, IsoImage *image,
     case S_IFLNK:
         {
             /* source is a symbolic link */
-            char dest[PATH_MAX];
+            char dest[LIBISOFS_NODE_PATH_MAX];
             IsoSymlink *link;
 
-            ret = iso_file_source_readlink(src, dest, PATH_MAX);
+            ret = iso_file_source_readlink(src, dest, LIBISOFS_NODE_PATH_MAX);
             if (ret < 0) {
                 break;
             }
@@ -216,6 +221,9 @@ int default_create_node(IsoNodeBuilder *builder, IsoImage *image,
         }
         iso_aa_get_acl_text(aa_string, info.st_mode, &a_text, &d_text,
                             1 << 15); /* free ACL texts */
+        if(aa_string != NULL)
+            free(aa_string);
+        aa_string = NULL;
     }
 
     /* Obtain ownership of eventual AAIP string */
@@ -226,6 +234,8 @@ int default_create_node(IsoNodeBuilder *builder, IsoImage *image,
         ret = iso_node_add_xinfo(new, aaip_xinfo_func, aa_string);
         if (ret < 0)
             return ret;
+    } else if(aa_string != NULL) {
+        free(aa_string);
     }
 
     *node = new;
@@ -236,6 +246,9 @@ int default_create_node(IsoNodeBuilder *builder, IsoImage *image,
 static
 void default_free(IsoNodeBuilder *builder)
 {
+    /* The .free() method of IsoNodeBuilder shall free private data but not
+       the builder itself. The latter is done in iso_node_builder_unref().
+    */
     return;
 }
 
