@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2009 - 2010 Thomas Schmitt
+ * Copyright (c) 2009 - 2011 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -329,6 +329,9 @@ int ecma119_writer_compute_data_blocks(IsoImageWriter *writer)
                              image start and not for the partition */;
 
     }
+
+    target->tree_end_block = target->curblock;
+
     return ISO_SUCCESS;
 }
 
@@ -380,11 +383,14 @@ void write_one_dir_record(Ecma119Image *t, Ecma119Node *node, int file_id,
         multi_extend = (node->info.file->nsections - 1 == extent) ? 0 : 1;
     } else {
         /*
-         * for nodes other than files and dirs, we set both
-         * len and block to 0
+         * for nodes other than files and dirs, we set len to 0, and
+         * the content block address to a dummy value.
          */
         len = 0;
-        block = 0;
+        if (! t->old_empty)
+            block = t->empty_file_block;
+        else
+            block = 0;
     }
 
     /*
@@ -871,6 +877,7 @@ int ecma119_writer_write_data(IsoImageWriter *writer)
 {
     int ret;
     Ecma119Image *t;
+    uint32_t curblock;
 
     if (writer == NULL) {
         return ISO_ASSERT_FAILURE;
@@ -888,6 +895,20 @@ int ecma119_writer_write_data(IsoImageWriter *writer)
         if (ret < 0) 
             return ret;
     }
+
+    curblock = (t->bytes_written / 2048) + t->ms_block;
+    if (curblock != t->tree_end_block) {
+        char msg[100];
+
+        sprintf(msg,
+                "Calculated and written ECMA-119 tree end differ: %lu <> %lu",
+                (unsigned long) t->tree_end_block,
+                (unsigned long) curblock);
+        iso_msgs_submit(0, msg, 0, "WARNING", 0);
+
+        t->tree_end_block = 1;/* Mark for harsher reaction at end of writing */
+    }
+
     return ISO_SUCCESS;
 }
 
@@ -1087,7 +1108,36 @@ int zero_writer_free_data(IsoImageWriter *writer)
 }
 
 static
-int zero_writer_create(Ecma119Image *target, uint32_t num_blocks)
+int tail_writer_compute_data_blocks(IsoImageWriter *writer)
+{
+    int ret;
+    Ecma119Image *target;
+    struct iso_zero_writer_data_struct *data;
+    char msg[160];
+
+    target = writer->target;
+    ret = iso_align_isohybrid(target, 0);
+    if (ret < 0)
+        return ret;
+    data = (struct iso_zero_writer_data_struct *) writer->data;
+    if (data->num_blocks != target->tail_blocks) {
+        sprintf(msg, "Aligned image size to cylinder size by %d blocks",
+                     target->tail_blocks - data->num_blocks);
+        iso_msgs_submit(0, msg, 0, "NOTE", 0);
+        data->num_blocks = target->tail_blocks;
+    }
+    if (target->tail_blocks <= 0)
+        return ISO_SUCCESS;
+    ret = zero_writer_compute_data_blocks(writer);
+    return ret;
+}
+
+/*
+  @param flag bit0= use tail_writer_compute_data_blocks rather than
+                    zero_writer_compute_data_blocks
+*/
+static
+int zero_writer_create(Ecma119Image *target, uint32_t num_blocks, int flag)
 {
     IsoImageWriter *writer;
     struct iso_zero_writer_data_struct *data;
@@ -1103,7 +1153,11 @@ int zero_writer_create(Ecma119Image *target, uint32_t num_blocks)
     }
     data->num_blocks = num_blocks;
 
-    writer->compute_data_blocks = zero_writer_compute_data_blocks;
+    if (flag & 1) {
+        writer->compute_data_blocks = tail_writer_compute_data_blocks;
+    } else {
+        writer->compute_data_blocks = zero_writer_compute_data_blocks;
+    }
     writer->write_vol_desc = zero_writer_write_vol_desc;
     writer->write_data = zero_writer_write_data;
     writer->free_data = zero_writer_free_data;
@@ -1135,6 +1189,8 @@ int write_vol_desc_terminator(Ecma119Image *target)
     uint8_t buf[BLOCK_SIZE];
     struct ecma119_vol_desc_terminator *vol;
 
+    memset(buf, 0, BLOCK_SIZE);
+
     vol = (struct ecma119_vol_desc_terminator *) buf;
 
     vol->vol_desc_type[0] = 255;
@@ -1209,7 +1265,7 @@ int write_head_part2(Ecma119Image *target, int *write_count, int flag)
         return ISO_SUCCESS;
 
     /* Write multi-session padding up to target->partition_offset + 16 */
-    memset(buf, 0, 2048);
+    memset(buf, 0, BLOCK_SIZE);
     for(; *write_count < target->partition_offset + 16; (*write_count)++) {
         res = iso_write(target, buf, BLOCK_SIZE);
         if (res < 0)
@@ -1342,7 +1398,7 @@ static
 void *write_function(void *arg)
 {
     int res, first_partition = 1, last_partition = 0, sa_type;
-    size_t i;
+    int i;
     IsoImageWriter *writer;
 
     Ecma119Image *target = (Ecma119Image*)arg;
@@ -1400,6 +1456,12 @@ void *write_function(void *arg)
        Eventually free target */
     ecma119_image_free(target);
 
+    if (target->tree_end_block == 1) {
+        iso_msgs_submit(0,
+    "Image is most likely damaged. Calculated/written block address mismatch.",
+                        0, "FATAL", 0);
+    }
+
 #ifdef Libisofs_with_pthread_exiT
     pthread_exit(NULL);
 #else
@@ -1575,6 +1637,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->hardlinks = opts->hardlinks;
     target->aaip = opts->aaip;
     target->always_gmt = opts->always_gmt;
+    target->old_empty = opts->old_empty;
     target->untranslated_name_len = opts->untranslated_name_len;
     target->allow_dir_id_ext = opts->allow_dir_id_ext;
     target->omit_version_numbers = opts->omit_version_numbers
@@ -1587,6 +1650,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->allow_full_ascii = opts->allow_full_ascii;
     target->relaxed_vol_atts = opts->relaxed_vol_atts;
     target->joliet_longer_paths = opts->joliet_longer_paths;
+    target->joliet_long_names = opts->joliet_long_names;
     target->rrip_version_1_10 = opts->rrip_version_1_10;
     target->rrip_1_10_px_ino = opts->rrip_1_10_px_ino;
     target->aaip_susp_1_10 = opts->aaip_susp_1_10;
@@ -1636,7 +1700,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
         system_area = src->system_area_data;
         system_area_options = src->system_area_options;
     } else {
-        system_area_options = opts->system_area_options & 0xfc;
+        system_area_options = opts->system_area_options & 0xfffffffc;
     }
     sa_type = (system_area_options >> 2) & 0x3f;
     if (sa_type != 0 && sa_type != 3)
@@ -1665,9 +1729,9 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->partition_secs_per_head = opts->partition_secs_per_head;
     target->partition_heads_per_cyl = opts->partition_heads_per_cyl;
     if (target->partition_secs_per_head == 0)
-        target->partition_secs_per_head = 63;
+        target->partition_secs_per_head = 32;
     if (target->partition_heads_per_cyl == 0)
-        target->partition_heads_per_cyl = 255;
+        target->partition_heads_per_cyl = 64;
     target->eff_partition_offset = 0;
     target->partition_root = NULL;
     target->partition_l_table_pos = 0;
@@ -1733,6 +1797,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->mipsel_p_filesz = 0;
 
     target->empty_file_block = 0;
+    target->tree_end_block = 0;
 
     target->wthread_is_running = 0;
 
@@ -1779,8 +1844,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     if (target->iso1999) {
         nwriters++;
     }
-    if (target->tail_blocks > 0) 
-        nwriters++;
+    nwriters++; /* Tail padding writer */
     if ((target->md5_file_checksums & 1) || target->md5_session_checksum) {
         nwriters++;
         image_checksums_mad = 1; /* from here on the loaded checksums are
@@ -1862,11 +1926,9 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
 
 
     /* IMPORTANT: This must be the last writer before the checksum writer */
-    if (target->tail_blocks > 0) {
-        ret = zero_writer_create(target, target->tail_blocks);
-        if (ret < 0)
-            goto target_cleanup;
-    }
+    ret = zero_writer_create(target, target->tail_blocks, 1);
+    if (ret < 0)
+        goto target_cleanup;
 
     if ((target->md5_file_checksums & 1) || target->md5_session_checksum) {
         ret = checksum_writer_create(target);
@@ -1920,8 +1982,15 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
         if (i == el_torito_writer_index)
     continue;
 
-        /* Exposing address of data start to IsoWriteOpts */
+        /* Exposing address of data start to IsoWriteOpts and memorizing
+           this address for all files which have no block address: 
+           symbolic links, device files, empty data files.
+           filesrc_writer_compute_data_blocks() and filesrc_writer_write_data()
+           will account resp. write this single block. 
+        */
         if (i == file_src_writer_index) {
+            if (! target->old_empty)
+                target->empty_file_block = target->curblock;
             opts->data_start_lba = target->curblock;
         }
 
@@ -2268,6 +2337,8 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         /* reader cancelled */
         return ISO_CANCELED;
     }
+    if (ret < 0)
+        return ret;
     if (target->checksum_ctx != NULL) {
         /* Add to image checksum */
         target->checksum_counter += count;
@@ -2279,7 +2350,7 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         return ret;
 
     /* total size is 0 when writing the overwrite buffer */
-    if (ret > 0 && (target->total_size != (off_t) 0)){
+    if (target->total_size != (off_t) 0){
         unsigned int kbw, kbt;
         int percent;
 
@@ -2296,7 +2367,7 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         }
     }
 
-    return ret;
+    return ISO_SUCCESS;
 }
 
 int iso_write_opts_new(IsoWriteOpts **opts, int profile)
@@ -2366,6 +2437,7 @@ int iso_write_opts_new(IsoWriteOpts **opts, int profile)
     wopts->ascii_disc_label[0] = 0;
     wopts->will_cancel = 0;
     wopts->allow_dir_id_ext = 0;
+    wopts->old_empty = 0;
     wopts->untranslated_name_len = 0;
 
     *opts = wopts;
@@ -2455,6 +2527,15 @@ int iso_write_opts_set_aaip(IsoWriteOpts *opts, int enable)
     return ISO_SUCCESS;
 }
 
+int iso_write_opts_set_old_empty(IsoWriteOpts *opts, int enable)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    opts->old_empty = enable ? 1 : 0;
+    return ISO_SUCCESS;
+}
+
 int iso_write_opts_set_untranslated_name_len(IsoWriteOpts *opts, int len)
 {
     if (opts == NULL) {
@@ -2561,6 +2642,15 @@ int iso_write_opts_set_joliet_longer_paths(IsoWriteOpts *opts, int allow)
     return ISO_SUCCESS;
 }
 
+int iso_write_opts_set_joliet_long_names(IsoWriteOpts *opts, int allow)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    opts->joliet_long_names = allow ? 1 : 0;
+    return ISO_SUCCESS;
+}
+
 int iso_write_opts_set_rrip_version_1_10(IsoWriteOpts *opts, int oldvers)
 {
     if (opts == NULL) {
@@ -2830,7 +2920,7 @@ int iso_write_opts_set_system_area(IsoWriteOpts *opts, char data[32768],
         memcpy(opts->system_area_data, data, 32768);
     }
     if (!(flag & 4))
-        opts->system_area_options = options & 0xff;
+        opts->system_area_options = options & 0x3ff;
     return ISO_SUCCESS;
 }
 
