@@ -8,7 +8,18 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -413,6 +424,22 @@ int checksum_cx_xinfo_func(void *data, int flag)
     return 1;
 }
 
+/* The iso_node_xinfo_cloner function which gets associated to
+ * checksum_cx_xinfo_func by iso_init() resp. iso_init_with_flag() via
+ * iso_node_xinfo_make_clonable()
+ */
+int checksum_cx_xinfo_cloner(void *old_data, void **new_data, int flag)
+{
+    *new_data = NULL;
+    if (flag)
+        return ISO_XINFO_NO_CLONE;
+    if (old_data == NULL)
+        return 0;
+    /* data is an int disguised as pointer. It does not point to memory. */
+    *new_data = old_data;
+    return 0;
+}
+
 
 /* Function to identify and manage md5 sums of unspecified providence stored
  * directly in this xinfo.
@@ -425,13 +452,29 @@ int checksum_md5_xinfo_func(void *data, int flag)
     return 1;
 }
 
+/* The iso_node_xinfo_cloner function which gets associated to
+ * checksum_md5_xinfo_func by iso_init() resp. iso_init_with_flag() via
+ * iso_node_xinfo_make_clonable()
+ */
+int checksum_md5_xinfo_cloner(void *old_data, void **new_data, int flag)
+{
+    *new_data = NULL;
+    if (flag)
+        return ISO_XINFO_NO_CLONE;
+    if (old_data == NULL)
+        return 0;
+    *new_data = calloc(1, 16);
+    if (*new_data == NULL)
+        return ISO_OUT_OF_MEM;
+    memcpy(*new_data, old_data, 16);
+    return 16;
+}
+
 
 /* ----------------------------------------------------------------------- */
 
 /* MD5 checksum image writer */
 
-#ifdef Libisofs_with_checksumS
-
 /*
   @flag bit0= recursion
         bit1= session will be appended to an existing image
@@ -508,15 +551,10 @@ int checksum_copy_old_nodes(Ecma119Image *target, IsoNode *node, int flag)
     return ISO_SUCCESS;
 }
 
-#endif /* Libisofs_with_checksumS */
-
 
 static
 int checksum_writer_compute_data_blocks(IsoImageWriter *writer)
 {
-
-#ifdef Libisofs_with_checksumS
-
     size_t size;
     Ecma119Image *t;
     int ret;
@@ -557,9 +595,6 @@ int checksum_writer_compute_data_blocks(IsoImageWriter *writer)
                                t->checksum_idx_counter + 2, 16, "MD5", 0);
     if (ret < 0)
         return ret;
-
-#endif /* Libisofs_with_checksumS */
-
     return ISO_SUCCESS;
 }
 
@@ -580,9 +615,6 @@ int checksum_writer_write_vol_desc(IsoImageWriter *writer)
 static
 int checksum_writer_write_data(IsoImageWriter *writer)
 {
-
-#ifdef Libisofs_with_checksumS
-
     int wres, res;
     size_t i, size;
     Ecma119Image *t;
@@ -649,12 +681,6 @@ ex:;
     if (ctx != NULL)
         iso_md5_end(&ctx, md5);
     return(res);
-
-#else /* Libisofs_with_checksumS */
-
-    return ISO_SUCCESS;
-
-#endif /* ! Libisofs_with_checksumS */
 }
 
 
@@ -684,17 +710,11 @@ int checksum_writer_create(Ecma119Image *target)
 
     /* add this writer to image */
     target->writers[target->nwriters++] = writer;
-
-#ifdef Libisofs_with_checksumS
-
     /* Account for superblock checksum tag */
     if (target->md5_session_checksum) {
         target->checksum_sb_tag_pos = target->curblock;
         target->curblock++;
     }
-
-#endif /* Libisofs_with_checksumS */
-
     return ISO_SUCCESS;
 }
 
@@ -702,9 +722,6 @@ int checksum_writer_create(Ecma119Image *target)
 static
 int iso_md5_write_scdbackup_tag(Ecma119Image *t, char *tag_block, int flag)
 {
-
-#ifdef Libisofs_with_checksumS
-
     void *ctx = NULL;
     off_t pos = 0, line_start;
     int record_len, block_len, res, i;
@@ -753,12 +770,6 @@ ex:;
     if (ctx != NULL)
         iso_md5_end(&ctx, md5);
     return res;
-
-#else
-
-    return ISO_SUCCESS;
-
-#endif /* Libisofs_with_checksumS */
 }
 
 
@@ -772,9 +783,6 @@ ex:;
  */
 int iso_md5_write_tag(Ecma119Image *t, int flag)
 {
-
-#ifdef Libisofs_with_checksumS
-
     int res, mode, l, i, wres, tag_id_len;
     void *ctx = NULL;
     char md5[16], tag_block[2048], *tag_id;
@@ -863,13 +871,6 @@ ex:;
     if (ctx != NULL)
         iso_md5_end(&ctx, md5);
     return res;
-
-#else /* Libisofs_with_checksumS */
-
-    return ISO_SUCCESS;
-
-#endif /* ! Libisofs_with_checksumS */
-
 }
 
 
